;----------------------------------------------------------------
; InPlainView Hidden Text Finder v0.1 (21 September 2002)
; Freeware, Open Source, GPL, Copyleft, whatever you want.
;----------------------------------------------------------------
;
; It's just a very small utility, quick-and-dirty coded,
; to recover text "hidden" by InPlainView steganography software 
;
; How to use: run my program. Select the BMP file with hidden 
; information data. That's all. Several things can happen:
;
;  - if the file is not a 24-bits BMP, you have an error message
;
;  - if the 5 first bytes of hidden data do not look like a
;    InPlainView hidden text header, you have an error message
;
;  - if the hidden text is not encrypted, it's displayed on
;    a MessageBox
;
;  - if the hidden text is encrypted, it's saved as a raw
;    binary file for you to analyze, called "hidden_encrypted.bin".
;    You may have a good chance to recover the text if it's long
;    enough, not random, and the password is short enough. The
;    encryption algorithm is a simple XOR of the plain text with
;    the password, repeated as many time needed.
;
; Guillermito
; guillermito@pipo.com
; http://www.guillermito.net
; September 18th 2002
;
; Assemble it with:
;      TASM32 /ml /m3 /z /t InPlainView_Hidden_Text_Finder
;      TLINK32 -Tpe -aa InPlainView_Hidden_Text_Finder,,,import32 


callW macro   x
extrn x:PROC
call x
endm

   .486
   .model flat

.data

openfilename_struct:
lStructSize       dd openfilename_struct_size
hwndOwner         dd 0
hInstance         dd 0
lpstrFilter       dd offset filter
lpstrCustomFilter dd 0
nMaxCustFilter    dd 0
nFilterIndex      dd 0
lpstrFile         dd offset namebuffer
nMaxFile          dd 255
lpstrFileTitle    dd 0
nMaxFileTitle     dd 32
lpstrInitialDir   dd 0
lpstrTitle        dd 0
Flags             dd 1000h+4h+200000h
nFileOffset       dw 0
nFileExtension    dw 0
lpstrDefExt       dd 0
lCustData         dd 0
lpfnHook          dd 0
lpTemplateName    dd 0
openfilename_struct_size equ $-offset openfilename_struct

filter            db "Bmp files *.bmp",0,"*.bmp",0,0
namebuffer	      db 255 dup(0)
result_title      db "InPlainView Hidden Text Finder by Guillermito", 0
no_bmp            db "This file does not have the 24-bits BMP signature",0
no_data           db "This BMP file does not seem to contain hidden data",0
encrypted         db "The hidden data is encrypted with a password.",0dh,0ah
                  db "Saving it as a raw binary file called hidden_encrypted.bin",0

file_name          db "hidden_encrypted.bin",0
file_handle2       dd ?
password_or_not    dd ?
file_size          dd ?
file_handle        dd ?
file_mem_buffer    dd ?
file_nb_bytes_read dd ?
results            dd ?
pixels_data_start  dd ?
size_hidden_data   dd ?

.code

programme:

;-------- choose a file -------------

push offset openfilename_struct
callW GetOpenFileNameA             ;choose file
test eax, eax
jz end

;-------- open it --------------

push 0
push 80h                          ;FILE_ATTRIBUTE_NORMAL
push 3                            ;OPEN_EXISTING
push 0
push 0
push 80000000h+40000000h          ;GENERIC_READ+GENERIC_WRITE
push dword ptr [lpstrFile]
callW CreateFileA                 ;open file
inc eax
jz end
dec eax
mov file_handle, eax

;---------- get its size ---------

push 0
push dword ptr file_handle
callW GetFileSize                 ;get file size in bytes
inc eax
jz close_file
dec eax
mov file_size, eax

;--------- alloc memory for file --------

mov eax, file_size
add eax, 1024
push eax                         ;number of bytes to allocate
push 40h                         ;40h=LMEM_ZEROINIT
callW LocalAlloc
test eax, eax
jz close_file
mov file_mem_buffer, eax
add eax, file_size
add eax, 1024
mov results, eax

;------- read entire file ------------

push 0
push offset file_nb_bytes_read
push file_size
push file_mem_buffer
push file_handle
callW ReadFile
test eax, eax
jz free_memory
mov eax, file_nb_bytes_read
cmp eax, file_size
jnz free_memory

;-------- find BMP signature ---------

mov esi, file_mem_buffer
lodsw
cmp ax, "MB"
jne trouble

;------- find number of bits per pixel ------

add esi, 8
lodsd
add eax, file_mem_buffer
mov pixels_data_start, eax
add esi, 14
lodsw
cmp ax, 24
je this_looks_like_a_bmp

;-------- not 24-bits BMP -----------

trouble:
push 0
push offset result_title
push offset no_bmp
push 0
callW MessageBoxA
jmp free_memory

;------ Extract hidden first 5 bytes -----------
; they should be: 
; dw 0
; dw size of hidden file
; db 0 if no password, 1 if password

this_looks_like_a_bmp:
mov ecx, 5
mov esi, pixels_data_start
mov edi, file_mem_buffer

extract_all:
push ecx
xor ebx, ebx
mov ecx, 8
extract_a_byte:
lodsb				;get byte
and al, 1			;eliminate everything except LSB
dec cl
rol al, cl			;put the bit where it belongs in the byte
inc cl
or bl, al			;along with its friends
loop extract_a_byte
xchg al, bl
stosb
pop ecx
loop extract_all
push esi

;------- check the 2 first bytes (0)-------

mov esi, file_mem_buffer
lodsw
cmp ax, 0
jne probably_no_hidden_data

;------- check the next 2 bytes (size) -------

xor eax, eax
lodsb
rol eax, 8
lodsb
cmp eax, file_size
ja probably_no_hidden_data
cmp eax, 0
je probably_no_hidden_data
mov size_hidden_data, eax

;------- check the next byte (0 or 1) -------

lodsb
mov bl, al
and al, 11111110b
cmp al, 0
je lets_go

;----- they do not look like InPlainView data -------

probably_no_hidden_data:
push 0
push offset result_title
push offset no_data
push 0
callW MessageBoxA
jmp free_memory

;------- they look like IPV data --------

lets_go:
cmp bl, 1
jne no_password

;----- there is a password: display message ---------

push 0
push offset result_title
push offset encrypted
push 0
callW MessageBoxA
mov password_or_not, 0ffh

;------ extract the hidden message ---------

no_password:
pop esi
mov ecx, size_hidden_data
mov edi, file_mem_buffer

extract_all2:
push ecx
xor ebx, ebx
mov ecx, 8
extract_a_byte2:
lodsb
and al, 1
dec cl
rol al, cl
inc cl
or bl, al
loop extract_a_byte2
xchg al, bl
stosb
pop ecx
loop extract_all2

mov eax, 0
stosb

;----- there was a password? -------

cmp password_or_not, 0ffh
jnz display_result

;------- open a new file on the disk ----------------

push 0                  ;handle of file with attributes to copy (0=Ludwig)
push 80h				;file attributes (80h=FILE_ATTRIBUTE_NORMAL)
push 2					;how to create (2=OPEN_ALWAYS)
push 0					;address of security descriptor (0=Ludwig)
push 0					;share mode (0=Prevents the file from being shared)
push 40000000h				;access (read-write) mode (40000000h=GENERIC_WRITE)
push offset file_name			;address of name of the file
call CreateFileA
mov file_handle2, eax

;------- write buffer in it ----------------

push 0
push offset file_nb_bytes_read			;address of number of bytes written 
push size_hidden_data				;number of bytes to write
push file_mem_buffer			;address of data to write to file
push file_handle2			;handle of file to write to
callW WriteFile

;------- close it ----------------

push file_handle2
callW CloseHandle
jmp free_memory

;-------- show the stuff -----------

display_result:
push 0
push offset result_title
push file_mem_buffer
push 0
callW MessageBoxA

;------- close memory and file ----------------

free_memory:
push file_mem_buffer
callW LocalFree

;----------- exit ----------

close_file:
push dword ptr file_handle
callW CloseHandle

end:
push -1
callW ExitProcess

end programme
